from datetime import datetime
import sys
import os

from bwikibot.api import Wiki, datetime2zulu
from bwikibot.ui import shift_block

SESSION_FILE = 'cli.session'

def main():
    run(*sys.argv)

def run(*argv):
    load_extensions()
    if len(argv) < 2:
        print('You should pass action:')
        show_doc(actions)
    else:
        actions[argv[1]](*argv[2:])

    
actions = {}

def action(name):
    ''' Decorator to register function as action '''
    def decorate(f):
        actions[name] = f
        return f
    return decorate

def load_extensions():
    ''' trigger action decorator in extensions '''
    from bwikibot import extensions

def show_doc(actions):
    maxname = max(len(k) for k in actions.keys())
    for action, function in actions.items():
        doc = function.__doc__ or ''.join((
            function.__module__, '.',
            function.__name__, ' has no docstring'
        ))
        print('{:{width}s} - {}'.format(
            action,
            shift_block(doc, maxname + 5),
            width=maxname,
        ))

def get_wiki():
    wiki = Wiki()
    wiki.session_file(SESSION_FILE)
    return wiki

@action('throttle')
def throttle(delay=None):
    ''' Set or get delay between queries '''
    wiki = get_wiki()
    if not delay:
        print(wiki.throttle)
    else:
        wiki.throttle = float(delay)
        wiki.save(SESSION_FILE)


@action('read')
def read_page(*args):
    ''' Print page which's name passed in param to stdout '''
    name = ' '.join(args).strip()
    if not name:
        print('Pass page name as last param')
        return
    wiki = get_wiki()
    print(wiki.page(name).read())


@action('autocomplete')
def autocomplete(*args):
    ''' Print list of page names which starts from passed prefix '''
    prefix = ' '.join(args)
    wiki = get_wiki()
    print('\n'.join(
        wiki.opensearch(prefix)
    ))

@action('login')
def login():
    ''' Login to site with domain passed in param '''
    try:
        logout()
    except OSError:
        pass
    get_wiki()


@action('endpoint')
def change_endpoint(*args):
    ''' Change endpoing but don't change cookies (for wiki families) '''
    url = ' '.join(args)
    wiki = get_wiki()
    wiki.set_endpoint(url)
    wiki.save(SESSION_FILE)


@action('logout')
def logout():
    ''' Delete session file '''
    os.remove(SESSION_FILE)

@action('write')
def write_page(*args):
    ''' Read text from stdio and save to page with name passed in param '''
    name = ' '.join(args)
    wiki = get_wiki()

    text = []
    for line in sys.stdin:
        text.append(line)

    wiki.page(name).write(''.join(text), 'cli editing')

@action('now')
def get_now():
    ''' Prints current ISO-formatted zulu time '''
    print(datetime2zulu(datetime.utcnow()))



'''
Скрипти для війни з ботами на cybportal.univ.kiev.ua/wiki

Деталі:
http://bunyk.wordpress.com/2012/03/27/cybwiki-episode-ii-attack-of-the-botes/
'''

from bwikibot.api import Wiki
from bwikibot.ui import ask_y_n
from bwikibot.cli import action

@action('cybwiki_stop_spam')
def main():
    ''' Block spammers and delete their creations on cybwiki'''
    wiki = get_wiki()
    kill_registered(wiki)
    #protect_all_pages(wiki)


def protect_all_pages(wiki):
    for i, page in enumerate(wiki.all_pages):
        print(i, page.title)
        page.protect('Закриваємо доступ на редагування анонімам')


def list_pages_for_moderation(wiki):
    pages = list(page.title for page in yield_suspected_pages(wiki))
    print('\n'.join(pages))
    with open('check_list.txt', 'w') as f:
        f.write('\n'.join(pages))


def get_wiki():
    wiki = Wiki('http://cybportal.univ.kiev.ua/w/api.php', throttle=1)
    wiki.session_file('cybwiki.session')
    return wiki


def yield_suspected_pages(wiki):
    human_pages = set(p.title for p in wiki.category('Люди').members())

    for n, page in enumerate(wiki.all_pages):
        last_contributor = next(page.contributors())
        print(n, page.title, last_contributor.name)
        trusted = last_contributor.userpage().title in human_pages
        if not trusted:
           yield page
        else:
            print("trusted")


def kill_registered(wiki):
    bot_pages = set(p.title for p in wiki.category('Боти').members())
    human_pages = set(p.title for p in wiki.category('Люди').members())

    def shoot_bot(bot):
        bot.userpage().write('{{Spambot}}', 'Попався!')
        bot_pages.add(bot.userpage().title)
        for contrib in bot.contributions():
            spam_page = wiki.page(contrib['title'])
            only_bots = True
            for user in spam_page.contributors():
                if user.userpage().title not in bot_pages:
                    only_bots = False
                    break
            if only_bots:
                spam_page.delete('Спам.')
        bot.block('Die spammers, die!')

    for user in wiki.all_users:
        userpage = user.userpage()
        if userpage.title in bot_pages:
            continue
        if userpage.title in human_pages:
            continue

        show_user_info(user)
        if user.edit_count() < 3:
            verdict = True
            print('Припускаємо що бот')
        else:
            #verdict = ask_y_n('Це бот, як думаєте? (y/n або порожній рядок якщо не знаєте)')
            verdict = True

        if verdict == '':
            continue
        elif verdict:
            shoot_bot(user)
        else:
            userpage.append('{{Approved}}',
                'Підтвердження того що власник сторінки - не бот.'
            )


def delete_pages(wiki, list_filename):
    names = open(list_filename, 'r', encoding='utf-8')
    for name in names:
        print('deleting %s' % name)
        page = wiki.page(name[:-1])
        if page.exists():
            page.delete('spam')


def show_user_info(user):
    userpage = user.userpage()
    page_text = userpage.read()
    if page_text:
        print(page_text)
    else:
        print('Сторінка користувача порожня')
    edits = user.edit_count()
    print(userpage.title, 'Редагувань: %d' % edits)



# (C) Code based on code of BotCat (watchcat.py) by Panther (ru.wikipedia)
# Distributed under the terms of the MIT license.

from datetime import timedelta, datetime
import re
import sys

from bwikibot.cli import get_wiki, action
from bwikibot.api import datetime2zulu, zulu2datetime, WikiError
from bwikibot.api import Change

from . import licenses


debug = False

wait_before_check = timedelta(hours=1, minutes=30)

user_welcome = '{{subst:welcome}}--~~~~\n\n'
problem_images_tag = '<!-- problem images list -->'

user_warning = '''
{{subst:Проблемні зображення}}
%%(images)s
%(tag)s
--~~~~
''' % {'tag': problem_images_tag}

image_issue = '* [[:%(image)s|%(image)s]]: %(summary)s\n'
usertalk_summary = 'Робот: попередження про проблеми з ліцензуванням зображень'
license_summaries= {
    'untagged': {
        'image': '{{subst:nld}}',
        'summary': 'Відсутня правова інформація',
    },
    'no_license': {
        'image': '{{subst:nld}}',
        'summary': 'Нема шаблону ліцензії',
    },
    'no_source': {
        'image': '{{subst:nsd}}',
        'summary': 'Не зазначене джерело',
    },
    'prohibited': {
        'image': '{{subst:nld}}',
        'summary': 'Використана заборонена ліцензія',
    },
    'old': {
        'image': '{{subst:nld}}',
        'summary': 'Використана застаріла ліцензія',
    },
    'no_template': {
        'image': '{{subst:nld}}',
        'summary': 'Ліцензія без шаблону',
    },
}

category_by_date = "Файли з нез'ясованим статусом від %(day)d %(month)s %(year)d"
month_names = [0, 'січня', 'лютого', 'березня', 'квітня', 'травня', 'червня', 'липня', 'серпня', 'вересня', 'жовтня', 'листопада', 'грудня']
category_content = "[[Категорія:Файли з нез'ясованим статусом|%(month)d-%(day)02d]]\n"
category_summary = 'Робот: автоматичне створення категорії'

# TODO: check for raw strings
infobox_regex = '\{\{\s*(Зображення|Зображення2)\s*\|.+?\}\}'
source_regex = '\{\{\s*(Зображення|Зображення2).+?Джерело\s*=\s*(?P<src>[^\|\}]+).+?\}\}'
author_regex = '\{\{\s*(Зображення|Зображення2).+?Автор\s*=\s*(?P<auth>[^\|\}]+)[\r\n\|].+?\}\}'
fake_source = '([Аа]втор|[Дд]жерело|[Іі]нтернет|[Ii]nternet|[Нн]евідом[ео]|[Uu]nknown|\?+)'
min_info_length = 8

@action('check_uploads')
def main(start_from, limit):
    ''' Check and mark new files for licensing issues,
        and send messages to uploaders.'''
    wiki = get_wiki()
    for upload in get_uploads(wiki, start_from):
        check_upload(upload, int(limit))

def get_uploads(wiki, start):
    if debug: # faking uploads
        yield Change(wiki, dict(
            title='Файл:Remainder-pattern1.png',
            timestamp=datetime2zulu(
                datetime.utcnow() - wait_before_check * 2
            ),
            user='BunykBot',
        ))
    else:
        start = zulu2datetime(start) if start else None
        for upload in wiki.logevents(
                event_type='upload', start=start,
                direction='newer',
        ):
            yield upload

counter = 0

def check_upload(upload, limit):
    global counter
    if upload.time + wait_before_check > datetime.utcnow():
        return
    print('Checking image {} uploaded at {} by {}'.format(
        upload.page.title,
        datetime2zulu(upload.time),
        upload.user.name,
    ))
    if not upload.page.exists():
        print('already deleted')
        return
    redirect = upload.page.redirect()
    if redirect:
        print('Redirect to:', redirect.title)
        return
    diagnosis = diagnose(upload.page.read())
    if diagnosis:
        print(diagnosis)
        summary = license_summaries[diagnosis]
        mark_image(upload.page, summary) 
        warn(upload.user, upload.page.title, summary['summary'])
        counter += 1
        if counter > limit:
            sys.exit(0)
    else:
        print('ok')

def mark_image(page, summary):
    check_category(page.wiki)
    try:
        page.write(
            (page.read() or '') + summary['image'],
            summary['summary']
        )
    except WikiError as e:
        print('Cannot mark image because of', e)

def check_category(wiki):
    if check_category.created:
        return
    now = datetime.utcnow()
    name = category_by_date % {
        'day': now.day,
        'month': month_names[now.month],
        'year': now.year
    }
    cat = wiki.category(name)
    if not cat.exists():
        cat.write(
            category_content % {
                'day': now.day,
                'month': now.month,
            },
            category_summary
        )
        print('Created', cat)
    check_category.created = True
check_category.created = False

def warn(user, image, problem):
    images = image_issue % {'image': image, 'summary': problem}
    talkpage = user.talkpage()
    talktext = talkpage.read() or user_welcome
    # check if user was warned
    pos = talktext.rfind(problem_images_tag)
    if pos >= 0:
        # check if there was new topics in talk
        pos2 = talktext.rfind('=', pos)
        if pos2 >= 0:
            # if where was - add full message to the end
            talktext += user_warning % {'images': images}
        else:
            # add new lines to old messages
            talktext = talktext[:pos] + images + talktext[pos:]
    else:
        # first warning
        talktext += user_warning % {'images': images}
    try:
        talkpage.write(talktext, usertalk_summary)
        print('User warned: ' + user.name)
    except Exception as e:
        print('User {} not warned because of {}'.format(user.name, e))

def get_regexp(text, flags=0, add_brackets=True):
    # Turn newlines to "|"
    text = re.sub('\s*[\n]+?', '|', text)

    if add_brackets:
        text = '\{\{\s*(Шаблон:)?\s*(' + text + ')\s*\}\}'
    return re.compile(text, flags)

self_licenses = get_regexp(licenses.self_licenses)
free_licenses = get_regexp(licenses.free_licenses)
unfree_licenses = get_regexp(licenses.unfree_licenses)
unfree_licenses_source = get_regexp(licenses.unfree_licenses_source, re.DOTALL)
old_licenses = get_regexp(licenses.old_licenses)
prohibited_licenses = get_regexp(licenses.prohibited_licenses)
warnings = get_regexp(licenses.warnings)
no_template = get_regexp(licenses.no_template, 0, False)

infobox = re.compile(infobox_regex, re.DOTALL)
source = re.compile(source_regex, re.DOTALL)
author = re.compile(author_regex, re.DOTALL)
fake = re.compile(fake_source, re.DOTALL)
comment = re.compile(r'<!\-\-.*?\-\->', re.DOTALL)

def simplify_wikilinks(text):
    ''' remove '|' characters '''
    return re.sub(r'\[\[(.+?)\|(.+?)\]\]', '[[1]]', text)

def diagnose(text_to_check):
    if not text_to_check:
        return 'untagged'
    text = text_to_check.strip()
    # remove comments
    text = comment.sub('', text).strip()
    if not text:
        return 'untagged'

    text = simplify_wikilinks(text)

    # searching for licenses
    is_self = self_licenses.search(text)
    is_free = free_licenses.search(text)
    is_unfree_source = unfree_licenses_source.search(text)
    is_warn = warnings.search(text) # already marked

    if is_self or is_warn:
        return False
    elif prohibited_licenses.search(text):
        return 'prohibited'
    elif is_free or is_unfree_source:
        # license need source
        if not has_source(text):
            return 'no_source'
    elif unfree_licenses.search(text):
        return False
    elif old_licenses.search(text):
        return 'old'
    else:
        if no_template.search(text):
            #Лицензия без шаблона
            return 'no_template'
        else:
            return 'no_license'

    return False

def has_source(text):
    #В этом шаблоне заполненен источник?
    is_source = source.findall(text)
    #В этом шаблоне заполненен автор?
    is_author = author.search(text)
    if infobox.search(text):
        if is_source:
            src_txt = ''
            for part in is_source:
                src_txt += part[1]
            #Удаляем из поля "источник" стандартные липовые фразы
            src_txt = fake.sub('', src_txt).strip()
        else:
            src_txt = ''
        if is_author:
            auth_txt = is_author.group('auth')
            #Удаляем из поля "автор" стандартные липовые фразы
            auth_txt = fake.sub('', auth_txt).strip()
        else:
            auth_txt = ''
        src_complex = src_txt + auth_txt
    else:
        #Удаляем из текста шаблоны и все заголовки
        src_complex = re.sub('\{\{.+?\}\}', '', text)
        src_complex = re.sub('=+.+?=+', '', src_complex).strip()
    #Если в итоге ничего не осталось, то считаем, что источника нет
    return len(src_complex) > min_info_length



from datetime import datetime
import time
import sys
import getpass
import json
import pickle
from pprint import pprint
from urllib.parse import urlencode
import mimetypes

import httplib2

DEBUG = False
SERVER_ENCODING = 'utf-8'
LIST_LIMIT = 400

httplib2.debuglevel = 4 if DEBUG else 0
http = httplib2.Http('.cache')


class WikiError(Exception):
    def __init__(self, code, msg):
        super(WikiError, self).__init__('WikiError: %s' % msg)
        self.code = code


def utf8(x):
    if not x:
        return b''
    if isinstance(x, bytes):
        return x
    return x.encode('utf-8')


def datetime2zulu(t):
    return t.isoformat().split('.')[0] + 'Z'

def zulu2datetime(z):
    return datetime.strptime(z, '%Y-%m-%dT%H:%M:%SZ')

def get_content_type(filename):
    return mimetypes.guess_type(filename)[0] or 'application/octet-stream'

def encode_multipart_formdata(fields, files):
    """
    fields is a sequence of (name, value) elements for regular form fields.
    files is a sequence of (name, filename, value) elements for data to be uploaded as files
    Return (content_type, body) ready for httplib.HTTP instance
    """
    BOUNDARY = '----------ThIs_Is_tHe_bouNdaRY_$'
    CRLF = b'\r\n'
    L = []
    def app(l):
        L.append(utf8(l))

    for (key, value) in fields:
        app('--' + BOUNDARY)
        app('Content-Disposition: form-data; name="%s"' % key)
        app('')
        app(value)
    for (key, filename, value) in files:
        app('--' + BOUNDARY)
        app('Content-Disposition: form-data; name="%s"; filename="%s"' % (key, filename))
        app('Content-Type: %s' % get_content_type(filename))
        app('')
        app(value)
    app('--' + BOUNDARY + '--')
    app('')
    body = CRLF.join(L)
    content_type = 'multipart/form-data; boundary=%s' % BOUNDARY
    return content_type, body

class Wiki():
    ''' Class for performing general requests to the API
    and storing session info.
    '''
    cookies = None
    namespaces_names = None # map from id to localised name
    namespaces_ids = None # map from canonical name to id

    def __init__(self, endpoint=None, throttle=1):
        ''' Create wiki client for given endpoint addres.  '''
        self._tokens = {}
        self.endpoint = endpoint
        self.throttle = throttle

        if endpoint:
            self.get_namespaces()
        # else may be load session

    def load(self, filename):
        ''' Load cookies, endpoint, namespaces. '''
        try:
            with open(filename, 'rb') as f:
                data = pickle.load(f)
                self.cookies = data['cookies']
                self.endpoint = data['endpoint']
                self.namespaces_names = data.get('namespaces_names')
                self.namespaces_ids = data.get('namespaces_ids')
                self.throttle = data.get('throttle', 1)
                self.init_namespaces()
        except IOError:
            return False
        return True

    def save(self, filename):
        ''' Save cookies, endpoint, namespaces '''
        with open(filename, 'wb') as f:
            data = dict(
                cookies=self.cookies,
                endpoint=self.endpoint,
                namespaces_names=self.namespaces_names,
                namespaces_ids=self.namespaces_ids,
                throttle=self.throttle,
            )
            pickle.dump(data, f)

    def request(self, params=None, method='GET', headers=None):
        ''' Perform any type of request to API.

        All query params should be passed throught params. 
        Authorisation cookies is passed throught headers
        authomathically if client is logged in.

        Returns headers and parsed json of resonce.
        '''
        assert self.endpoint, "Can't make a request without endpoint"
        if params is None:
            params = {}
        params['format'] = 'json'
        if headers is None:
            headers = {}
        headers['user-agent'] = 'bwikibot'
        if self.cookies:
            headers['cookie'] = '; '.join(
                '%s=%s' % (n, v) for n,v in self.cookies.items()
            )
        body = urlencode(params)
        if method == 'GET':
            query = self.endpoint + '?' + body
            body = None
        else:
            query = self.endpoint
            if not headers.get('Content-type'):
                headers['Content-type'] = 'application/x-www-form-urlencoded'
            if params['action'] == 'upload':
                data = params.pop('file')
                content_type, body = encode_multipart_formdata(
                    params.items(), 
                    [('file', params['filename'], data)]
                )
                headers['Content-type'] = content_type
        time.sleep(self.throttle)
        responce, content = http.request(
            query,
            method,
            headers=headers,
            body=body,
        )
        if DEBUG:
            print('\n')
        content = json.loads(content.decode(SERVER_ENCODING))
        if not content:
            raise WikiError('empty responce', 'MediaWiki empty responce: %s' % content)
        if not isinstance(content, dict):
            return content
        error = content.get('error')
        if error:
            print('Request params:')
            pprint(params)
            raise WikiError(error['code'], 'MediaWiki responce error: "%s"' % error['info'])

        return responce, content

    def query(self, params):
        '''Get query to the API.
        '''
        params['action'] = 'query'
        responce, content = self.request(params)
        return content['query']

    def query_cont(self, params):
        ''' Get query to the API for lists.

        Returns pair of responce and continue query params.
        '''
        params['action'] = 'query'
        responce, content = self.request(params)
        return content['query'], content.get('query-continue')

    def opensearch(self, search, limit=20):
        ''' Return a sequence of page titles which starts from given prefix '''
        responce, content = self.request(dict(
            action='opensearch',
            search=search,
            limit=limit,
        ))
        return content

    def _get_login_cookie(self, login, password):
        ''' Returns dictionary with cookies for given account.  '''
        responce, content = self.request(
            dict(
                action='login',
                lgname=login,
                lgpassword=password,
            ),
            method='POST'
        )
        content = content['login']
        if content['result'] == 'NeedToken':
            headers = {
                'Cookie': responce['set-cookie']
            }
            responce, content = self.request(
                dict(
                    action='login',
                    lgname=login,
                    lgpassword=password,
                    lgtoken=content['token']
                ),
                method='POST',
                headers=headers
            )
            content = content['login']
        if content['result'] == 'Success':
            pref = content['cookieprefix']
            cookies = {
                pref + '_session': content['sessionid'],
                pref + 'Token': content['lgtoken'],
                pref + 'UserName': content['lgusername'],
                pref + 'UserID': content['lguserid'],
                'path': '/',
            }
            return cookies
        else:
            raise WikiError(content['result'], 'Login error: %s.' % content['result'])

    def set_endpoint(self, url):
        self.endpoint = 'http://%s/w/api.php' % url

    def session_login(self, endpoint=None, login=None, password=None):
        ''' Start session for given account. '''
        if self.endpoint:
            self.set_endpoint(endpoint)
        else:
            if not endpoint:
                endpoint = input('Endpoint: ')
            self.set_endpoint(endpoint)

        if not login:
            login = input('Login: ')
        if not password:
            password = getpass.getpass('Password for %s: ' % login)
        try:
            self.cookies = self._get_login_cookie(login, password)
            self.get_namespaces()
        except WikiError as e:
            if e.code == 'WrongPass':
                print('Wrong password')
                sys.exit(0)
            else:
                raise

    def session_file(self, filename):
        ''' Start session which is persistent in file. '''
        if not self.load(filename):
            self.session_login()
            self.save(filename)
        
    def get_token(self, intoken, **params):
        ''' Get token for edit operation. '''
        token = self._tokens.get(intoken)
        if not token:
            params['intoken'] = intoken
            params['prop'] = 'info'
            res = self.query(params)
            try:
                token = list(res['pages'].values())[0]['%stoken' % intoken]
            except KeyError:
                pprint(res)
            self._tokens[intoken] = token
        return token

    def get_users(self,
        from_name=None, to_name=None,
        prefix=None, prop=None,
        with_edits_only=False,
        active_only=False,
    ):
        ''' Returns generator of users of current wiki.

        Every user is of type User.
        '''
        q = {
            'list': 'allusers',
            'aulimit': LIST_LIMIT,
        }
        if from_name:
            q['aufrom'] = from_name
        if to_name:
            q['auto'] = to_name
        if prefix:
            q['auprefix'] = prefix
        if prop:
            q['auprop'] = prop
        if active_only:
            q['auactiveusers'] = 1
        if with_edits_only:
            q['auwitheditsonly'] = 1
        res, cont = self.query_cont(q)
        for user in res['allusers']:
            yield User(self, user['name'])

        while cont:
            q.update(cont['allusers'])
            res, cont = self.query_cont(q)
            for user in res['allusers']:
                yield User(self, user['name'])

    @property
    def all_users(self):
        ''' Generator of all users of current wiki
        '''
        return self.get_users()

    @property
    def all_pages(self):
        for namespace in self.namespaces_ids.values():
            if namespace < 0:
                continue
            for page in self.get_pages(namespace=namespace):
                yield page

    def get_pages(self,
        from_name=None, to_name=None,
        prefix=None, prop=None,
        namespace=None
    ):
        q = {
            'list': 'allpages',
            'aplimit': LIST_LIMIT,
        }
        if from_name:
            q['apfrom'] = from_name
        if to_name:
            q['apto'] = to_name
        if prefix:
            q['apprefix'] = prefix
        if namespace:
            q['apnamespace'] = namespace

        res, cont = self.query_cont(q)
        for page in res['allpages']:
            yield Page(self, page['title'])

        while cont:
            q.update(cont['allpages'])
            res, cont = self.query_cont(q)
            for page in res['allpages']:
                yield Page(self, page['title'])

    def logevents(self, start=None, end=None,
        event_type=None,
        direction='older', # or newer
    ):
        q = {
            'list': 'logevents',
            'lelimit': LIST_LIMIT,
            'ledir': direction,
        }
        if start and end and start < end:
            start, end = end, start # newer changes first, remember?
        if start:
            q['lestart'] = datetime2zulu(start)
        if end:
            q['leend'] = datetime2zulu(end)
        if event_type:
            q['letype'] = event_type

        q['leprop'] = 'user|comment|timestamp|title'
        res, cont = self.query_cont(q)
        for event in res['logevents']:
            yield Change(self, event)

        while cont:
            q.update(cont['logevents'])
            res, cont = self.query_cont(q)
            for event in res['logevents']:
                yield Change(self, event)
        

    def recent_changes(self, start=None, end=None,
        change_type=None, # edit, new, log
        namespace=None,
    ):
        q = {
            'list': 'recentchanges',
            'rclimit': LIST_LIMIT,
            'rcdir': 'older', # newer changes first
        }
        if start and end and start < end:
            start, end = end, start # newer changes first, remember?
        if start:
            q['rcstart'] = datetime2zulu(start)
        if end:
            q['rcend'] = datetime2zulu(end)
        if namespace:
            q['rcnamespace'] = namespace
        if change_type:
            q['rctype'] = change_type
        q['rcprop'] = 'user|comment|timestamp|title|flags'
        res, cont = self.query_cont(q)
        for change in res['recentchanges']:
            yield Change(self, change)

        while cont:
            q.update(cont['recentchanges'])
            res, cont = self.query_cont(q)
            for change in res['recentchanges']:
                yield Change(self, change)

    def page(self, title):
        return Page(self, title)

    def category(self, title):
        return Category(self, title)

    def file(self, title):
        return File(self, title)

    def user(self, name):
        return User(self, name)

    def get_namespaces(self):
        ''' Init two maps for namespaces:
        namespaces_names: from id to localised name 
        namespaces_ids: from canonical name to id
        '''
        res = self.query(dict(
            meta='siteinfo',
            siprop='namespaces',
        ))
        self.namespaces_names = {}
        self.namespaces_ids = {}
        for ns in res['namespaces'].values():
            self.namespaces_names[ns['id']] = ns['*']
            self.namespaces_ids[ns.get('canonical')] = ns['id']

        self.init_namespaces()

    def init_namespaces(self):
        ''' Set some constants '''
        def ns_name(canonical):
            return self.namespaces_names[self.namespaces_ids[canonical]]

        self.USER_NS_NAME = ns_name('User')
        self.USER_TALK_NS_NAME = ns_name('User talk')
        self.CATEGORY_NS_NAME = ns_name('Category')
        self.FILE_NS_NAME = ns_name('File')

    def logged_user_info(self):
        ''' Get info about currently logged user.
        '''
        return self.query(
            dict(
                meta='userinfo',
                uiprop='groups|rights|editcount|email',
            ),
        )['userinfo']

    def __str__(self):
        return 'Wiki("%s")' % self.endpoint


class Change:
    def __init__(self, wiki, data):
        self.wiki = wiki
        self.type = data.get('type')
        self.time = zulu2datetime(data['timestamp'])
        self.page = Page(wiki, data['title'])
        self.user = User(wiki, data['user'])

    def __str__(self):
        return '{}.change({})'.format(self.page, self.time)

class Page:
    def __init__(self, wiki, title):
        self.wiki = wiki
        self.title = title
        self.pageid = None
        self.missing = None
        self.text = None

    def read(self):
        ''' Returns page text, and remembers page meta info, 
        such as namespace, normalized title, is page missing,
        and page id.
        '''
        if self.text:
            return self.text
        res = self.wiki.query(dict(
            prop='revisions',
            rvprop='content',
            titles=self.title,
        ))
        page = list(res['pages'].values())[0]
        self.namespace = page['ns']
        self.title = page['title']
        if 'missing' in page:
            self.missing = True
            return None
        else:
            self.missing = False
            self.pageid = page['pageid']
        self.text = page['revisions'][0]['*']
        return self.text

    def exists(self):
        self.read()
        return not self.missing

    def is_category(self):
        if not getattr(self, 'namespace', False):
            self.read()
        return self.namespace == self.wiki.namespaces_ids['Category']

    def write(self, text, summary):
        ''' Replace page contents with given text. 
        summary param is the description of edit.
        '''
        responce, content = self.wiki.request(
            dict(
                action='edit',
                title=self.title,
                text=text,
                token=self.wiki.get_token('edit', titles=self.title),
                summary=summary,
            ),
            method='POST',
        )
        self.text = text
        return content

    def append(self, message, summary): 
        body = self.read()
        if not body:
            body = ''
        return self.write(body + message, summary)

    def delete(self, reason):
        if not self.exists():
            return
        responce, content = self.wiki.request(
            dict(
                action='delete',
                title=self.title,
                token=self.wiki.get_token('delete', titles=self.title),
                reason=reason,
            ),
            method='POST',
        )

    def revisions(self):
        q = {
            'prop': 'revisions',
            'rvlimit': LIST_LIMIT / 10,
            'rvprop': 'user|timestamp|comment|content',
            'titles': self.title,
        }
        res, cont = self.wiki.query_cont(q)
        res = res['pages'].popitem()[1]
        if 'revisions' not in res:
            pprint(res)
            return
        for contrib in res['revisions']:
            yield contrib
        while cont:
            q.update(cont['revisions'])
            res, cont = self.wiki.query_cont(q)
            for contrib in res['revisions']:
                yield contrib


    def contributors(self):
        q = {
            'prop': 'revisions',
            'rvlimit': LIST_LIMIT,
            'rvprop': 'user',
            'titles': self.title,
        }
        res, cont = self.wiki.query_cont(q)
        res = res['pages'].popitem()[1]
        if 'revisions' not in res:
            pprint(res)
            return
        for contrib in res['revisions']:
            if 'user' in contrib:
                yield User(self.wiki, contrib['user'])
        while cont:
            q.update(cont['revisions'])
            res, cont = self.wiki.query_cont(q)
            for contrib in res['revisions']:
                if 'user' in contrib:
                    yield User(self.wiki, contrib['user'])

    def interwikis(self):
        ''' Returns dictionary of page names in other languages
        '''
        resp = self.wiki.query({
            'prop': 'langlinks',
            'titles': self.title,
            'lllimit': LIST_LIMIT,
        })
        pages = resp.get('pages')
        if not pages:
            return {}
        links = pages.popitem()[1].get('langlinks')
        if not links:
            return {}
        res = {}
        for link in links:
            res[link['lang']] = link['*']
        return res

    def redirect(self):
        ''' Return None or page to which it redirects
        '''
        redirects = self.wiki.query({
            'titles': self.title,
            'redirects': None,
        }).get('redirects', [])
        for r in redirects:
            if r['from'] == self.title:
                return self.wiki.page(r['to'])

    def protect(self, reason):
        req = dict(
            action='protect',
            title=self.title,
            token=self.wiki.get_token('protect', titles=self.title),
            reason=reason,
            protections='edit=autoconfirmed|move=autoconfirmed'
        )
        responce, content = self.wiki.request(req, method='POST')
        return content

    def __str__(self):
        return '%s.page("%s")' % (self.wiki, self.title)


class Category(Page):
    def __init__(self, wiki, title):
        if not title.startswith(wiki.CATEGORY_NS_NAME + ':'):
            title = wiki.CATEGORY_NS_NAME + ':' + title
        super(Category, self).__init__(wiki, title)

    def members(self):
        q = {
            'list': 'categorymembers',
            'cmlimit': LIST_LIMIT,
            'cmtitle': self.title,
        }
        res, cont = self.wiki.query_cont(q)
        for member in res['categorymembers']:
            yield self.wiki.page(member['title'])

        while cont:
            q.update(cont['categorymembers'])
            res, cont = self.wiki.query_cont(q)
            for member in res['categorymembers']:
                yield self.wiki.page(member['title'])

class File(Page):
    def __init__(self, wiki, title):
        if not title.startswith(wiki.FILE_NS_NAME + ':'):
            title = wiki.FILE_NS_NAME + ':' + title
        super(File, self).__init__(wiki, title)

    @property
    def filename(self):
        return self.title[len(self.wiki.FILE_NS_NAME) + 1:]

    def upload(self, path_to_file, comment, page_text):
        data = open(path_to_file, 'rb').read()
        responce, content = self.wiki.request(
            dict(
                action='upload',
                filename=self.filename,
                file=data,
                text=page_text,
                comment=comment,
                ignorewarnings=None,
                token=self.wiki.get_token('edit', titles='Головна сторінка'),
            ),
            method='POST',
        )
        res = content['upload']['result']
        if res == 'Warning':
            print(content['upload']['warnings'])
            return False
        if res == 'Success':
            return True

    def _url(self, width=None):
        query = dict(
            titles=self.title,
            prop='imageinfo',
            iiprop='url',
        )
        if width:
            query['iiurlwidth'] = width
        resp = self.wiki.query(query)
        info = list(resp['pages'].values())[0]['imageinfo'][0]
        return info['thumburl' if width else 'url']

    def download(self, to_file, width=None):
        headers, content = http.request(self._url(width))
        with open(to_file, 'wb') as f:
            f.write(content)

class User():
    def __init__(self, wiki, name):
        self.wiki = wiki
        self.name = name

    def __str__(self):
        return '%s.user("%s")' % (self.wiki, self.name)

    def userpage(self):
        return Page(self.wiki, '%s:%s' % (
            self.wiki.USER_NS_NAME,
            self.name
        ))

    def talkpage(self):
        return Page(self.wiki, '%s:%s' % (
            self.wiki.USER_TALK_NS_NAME,
            self.name
        ))

    def contributions(self):
        q = {
            'list': 'usercontribs',
            'uclimit': LIST_LIMIT,
            'ucuser': self.name,
        }
        res, cont = self.wiki.query_cont(q)
        for contrib in res['usercontribs']:
            yield contrib

        while cont:
            q.update(cont['usercontribs'])
            res, cont = self.wiki.query_cont(q)
            for contrib in res['usercontribs']:
                yield contrib

    def edit_count(self):
        return int(self.wiki.query({
            'list': 'users',
            'ususers': self.name,
            'usprop': 'editcount',
        })['users'][0]['editcount'])

    def is_anonymous(self):
        return not bool(self.wiki.query({
            'list': 'users',
            'ususers': self.name,
            'usprop': 'registration',
        })['users'][0].get('registration', False))

    def block(self, reason, nocreate=True, autoblock=True):
        req = dict(
            action='block',
            user=self.name,
            token=self.wiki.get_token('block', titles=self.name),
            reason=reason,
        )
        if nocreate:
            req['nocreate'] = ''
        if autoblock:
            req['autoblock'] = ''
        try:
            responce, content = self.wiki.request(req, method='POST')
            return content
        except WikiError as e:
            if e.code == 'alreadyblocked':
                pass
            else:
                raise

    def unblock(self, reason):
        self.wiki.request(
            dict(
                action='unblock',
                user=self.name,
                token=self.wiki.get_token('unblock', titles=self.name),
                reason=reason,
            ),
            method='POST',
        )

    def __str__(self):
        return '%s.user("%s")' % (self.wiki, self.name)

from difflib import ndiff

def copy_file(from_wiki, to_wiki, file_name):
    import uuid
    import os
    temp_fn = str(uuid.uuid1())
    source = from_wiki.file(file_name)
    source.download(temp_fn)
    to_wiki.file(file_name).upload(temp_fn,
        'test file exporting', source.read()
    )
    os.remove(temp_fn)

if __name__ == '__main__':
    cybwiki = Wiki('http://cybportal.univ.kiev.ua/w/api.php')
    for i, change in enumerate(cybwiki.recent_changes()):
        if i > 10:
            break
        print(change)



from datetime import datetime
from unittest import TestCase, main

from .api import datetime2zulu, zulu2datetime

class Zulu(TestCase):
    def test_now(self):
        now = datetime.now().replace(microsecond=0)
        zulu = datetime2zulu(now)

        self.assertEqual(zulu2datetime(zulu), now)


if __name__=='__main__':
    main()



from distutils.core import setup

setup(
    name = 'bwikibot',
    packages = ["bwikibot", 'bwikibot.extensions'],
    version = "0.3.8",
    description = "Simple mediawiki robot",
    author = "Taras Bunyk",
    author_email = "tbunyk@gmail.com",
    url = "http://code.google.com/p/bunykwikibot/",
    license='LICENSE.txt',
    #download_url = "link to tgz",
    classifiers = [
        "Programming Language :: Python",
        "Programming Language :: Python :: 3",
        "Development Status :: 2 - Pre-Alpha",
        "Environment :: Other Environment",
        "Intended Audience :: Developers",
        "Intended Audience :: Science/Research",
        "Intended Audience :: Education",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
        "Topic :: Software Development :: Libraries :: Python Modules",
        "Topic :: Text Processing :: Linguistic",
        "Topic :: Internet :: WWW/HTTP",
    ],
    long_description = open('README.txt').read(),
    install_requires=[
        "httplib2 >= 0.7.0",
    ],
    entry_points=dict(console_scripts=[
        'bwikibot=bwikibot.cli:main',
    ]),
)



# coding=utf-8

# Користувач:BotCat/Вільні ліцензії без джерела
self_licenses = r'''
PD\-[Ss]elf(\s*\|.*?)?
GFDL\-[Ss]elf(\s*\|.*?)?
[Ss]elf(2)?\s*\|\s*(GFDL|PD|PU|CC(\-BY)?(\-SA)?(\-\d\.\d)?|[Cc]c(\-by)?(\-sa)?(\-\d\.\d)?|cc\-by\-sa\-2\.5\,2\.0\,1\.0)
[Ss]elf(2)\s*\|\s*(GFDL|PD|PU|CC(\-BY)?(\-SA)?(\-\d\.\d)?(\,2\.0\,1\.0)?|[Cc]c(\-by)?(\-sa)?(\-\d\.\d)?)\s*\|\s*(GFDL|PD|PU|CC(\-BY)?(\-SA)?(\-\d\.\d)?|[Cc]c(\-by)?(\-sa)?(\-\d\.\d)?(\,2\.0\,1\.0)?)
[Ss]elf(2)\s*\|\s*(GFDL|PD|PU|CC(\-BY)?(\-SA)?(\-\d\.\d)?(\,2\.0\,1\.0)?|[Cc]c(\-by)?(\-sa)?(\-\d\.\d)?)\s*\|\s*(GFDL|PD|PU|CC(\-BY)?(\-SA)?(\-\d\.\d)?|[Cc]c(\-by)?(\-sa)?(\-\d\.\d)?(\,2\.0\,1\.0)?)
[Ss]elf(2)\|\GFDL\|\cc-by-sa-3.0,2.5,2.0,1.0
[Ss]elf(2)?\s*\|\s*(GFDL|CC(\-BY)?(\-SA)?(\-\d\.\d)?|[Cc][Cc](\-by)?(\-sa)?(\-\d\.\d)?|CC\-BY\-SA\-(3\.0\,)?2\.5\,2\.0\,1\.0|[Cc]c\-by\-sa\-(3\.0\,)?2\.5\,2\.0\,1\.0)\s*\|\s*(GFDL|CC(\-BY)?(\-SA)?(\-\d\.\d)?|[Cc]c(\-by)?(\-sa)?(\-\d\.\d)?|CC\-BY\-SA\-(3\.0\,)?2\.5\,2\.0\,1\.0|[Cc]c\-by\-sa\-(3\.0\,)?2\.5\,2\.0\,1\.0)
PD\-because\s*\|.+?
PD\-user\s*\|.+?
PD\-user1\s*\|.+?
PD\-user\-(w|en|de|he)\s*\|.+?
PD\-(ineligible|trivial)
PD\-textlogo
[Pp][Dd]-[Oo]ld
PD\-signature
'''

# Користувач:BotCat/Вільні ліцензії з джерелом
free_licenses = r'''
GFDL
GFDL\s*\|.+?
GFDL\-user\s*\|.+?
GFDL\-user\-(w|en|de|ja|ru)\s*\|.+?
GFDL\-OpenGeoDB
GNU[ _]FDL

PD\-([Oo]ld|[Aa]rt)
PDart
PD\-author\s*\|.+?
PD\-(Russia|Ukraine|Sweden|Argentina)
PD\-(RU|UA|AM|BY)\-exempt
PD\-Old\-Russia(1953)?
PD\-кадр
PD\-герб
PD\-Russia
PD\-UA
PD\-BY
PD\-India
PD\-BritishGov
[Гг]ербУА

PD\-Ital(y|ia)
PD\-Japan
PD\-USGov(\-CIA\-WF|\-CIA\-WF\-RU|\-NASA|\-NOAA|\-DOE|\-POTUS)?
PD\-US(\-JPL|\-patent|\-flag)?
[Pp]D\-USGov\-(Interior\-USGS|Military\-Badge)
PD\-GWPDA
PD\-NASA
[Pp]D\-USGov\-DOJ
Патент[ _]США
PD\-Argentina

GPL
LGPL
FAL
CC\-BY(\-SA)?(\-\d\.\d)?
CC\-SA(\-BY)?(\-\d\.\d)?
CC\-BY\-SA\-3\.0\,2\.5\,2\.0\,1\.0
CC\-by\-2\.0
[Cc][Cc]\-by\-3\.0
[Cc]c\-by\-2\.[05]
[Cc]c\-by\-sa\-3\.0\,2\.5\,2\.0\,1\.0
[Cc]c\-sa\-1\.0
[Cc]c\-by\-sa\-2\.[05]
[Cc]c\-by\-sa
[Cc]c\-by\-2\.0
[Сc]с\-by\-sa\-2\.5\,2\.0\,1\.0
[Cc][Cc]\-by\-sa\-3\.0
[Cc]c\-by\-nc\-sa\-2\.0\-dual
[Cc]c\-by\-sa\-2\.1\-jp
[Cc][Cc]\-[Zz]ero

[Kk]remlin\.ru\-image
[Ff]ree[ _]?use.*?
[Uu]se[ _]Free
[Ff]ree[ _]use[ _]provided[ _]that\s*\|.+?
[Cc]opyrightedFreeUseProvidedThat\s*\|.+?
[Cc]opyrightedFreeUse
[Aa]ttribution
[Nn]oRightsReserved

[Бб]ілецький
[Вв]ласенко
'''

# Користувач:BotCat/Невільні ліцензії без джерела
unfree_licenses = r'''
[Гг]азета
[Гг]ерби?
[Гг]роші
[Ее]мблема
[Зз]німок([ _](екран[ау]|сайту|[Вв]ікі(педії)?|програми|гри))?
[Зз]німок[ _]екран[ау][ _]сайту
[Кк]адр
[Кк]адр2.*?
[Лл]ого(тип)?
[Оо]бкладинка
[Оо]бкладинка[ _]альбому
[Оо]бкладинка[ _](фільму|dvd|DVD|відеокасети|комп'ютерної[ _]гри|гри)
[Оо]бкладинка[ _](книги|журналу|журналу[ _]Time)
[Оо]лімпійська медаль
[Пп]лакат
[Пп]остер2.*?
[Пп]остер
[Пп]оп[ _\-]музика
[Бб]люз:Аудіо
[Пп]рапори?
[Пп]оштова[ _]марка
[Пп]рототип
[Сс]криншот
[Тт]итульний[ _]аркуш
[Фф]отопортрет[ _]мерця
[Фф]отопортрет[ _]померлого
[Фф]П

([Aa]lbum|[Dd]vdtape|DVD|[Ff]ilm|[Gg]ame)cover
([Aa]lbum|[Dd]vdtape|DVD|[Ff]ilm|[Gg]ame)[ _]cover
([Bb]ook|[Mm]agazine|[Tt]ime[ _]magazine)[ _]?cover
[Cc]oatofarms
([Ff]ilm|[Gg]ame|[Ss]oftware|[Tt]v)\-screenshot
[Ff]lag
[Ii]con
[Ll]ogo
[Mm]ac\-software\-screenshot
[Mm]oney
[Nn]on\-free[_ ]3D[_ ]art
[Nn]on\-free[_ ]album[_ ]cover
[Nn]on\-free[_ ]logo
[Nn]on\-free[ _]poster
[Nn]on\-free[ _]historic[ _]image
[Nn]on\-free[ _]computer[ _]icon
[Nn]on\-free[ _]software[ _]screenshot
[Nn]on\-free[_ ]currency
[Oo]lympic[ _]medal
[Pp]oster
[Ss]hot
[Ss]creenshot([ _]po)?
[Ss]oftwarecover
[Ss]tamp
[Tt]v\-program\-logo
[Vv]ideo(cover|pcgame)?
[Ww]ikipedia\-screenshot
[Ww]ebpromo


[Cc]opyrightByWikimedia
'''

# Користувач:BotCat/Невільні ліцензії з джерелом
unfree_licenses_source = '''
[Ff]air[ _]?use[ _]?in\s*\|.+?
[Aa]rt
[Рр]исунок
[Фф]отографія[ _]скульптури
[Aa]rt[ _]sculpture
[Cc]opyrightedFreeUseProvidedThat\s*\|.+?
[Nn]on\-free[_ ]promotional
[Nn]on\-free[_ ]Olympics[_ ]logo

[Cc]haracter\-artwork

[Іі]сторичне[ _]фото
[Hh]istorical

[Кк]арта

[Сc]кульптура
[Nn]on\-free[_ ]3D[_ ]art
[Nn]on\-free[_ ]audio[_ ]sample

[Mm]usic[ _]sample
[Сс]емпл

[Рр]ок[ _]музика\s*\|.+?
[Мм]узика\s*\|.+?
[Пп]оп\-музика\s*\|.+?


[Ss]heet[ _]music
[Нн]отний[ _]приклад
'''

# Користувач:BotCat/Застарілі ліцензії
old_licenses = '''
[Ff]air[ _]?use
[Ff]air[ _]?old
[Pp]romo[Pp]hoto
[Pp]romotional
[Pp]romotional[ _]\-[ _]товар
PD
PU
GFDL\-en
[Ff]airuse[ _]\-[ _]архітектура
[Ff]airuse[ _]\-[ _]портрет
[Ff]airuse[ _]\-[ _]предмет
[Ff]airuse[ _]\-[ _]схема
'''


# Користувач:BotCat/Заборонені ліцензії
prohibited_licenses = '''
[Зз]аборонена[ _]ліцензія
CC[ _]bad
[Nn]oncommercial
CC.*?\-NC.*?
CC.*?\-ND.*?
[Cc]c.*?\-nc.*?
[Cc]c.*?\-nd.*?
[Ss]oviet\-?PU
PD[\- _]?[Ss]oviet
[Ss]oviet[ _](pd|PD)
PD\-USSR
[Ff]airuse[ _]\-[ _]карта
[Ww]ith[ _]permission
[Pp]ermission
PD\-meteor(ite)?
'''


# Користувач:BotCat/Попередження
warnings = '''
[Dd]isputed.*?
NCT.*?
RCTC.*?
[Зз]аміна[ _]з[ _]Вікісховища.*?
[Nn]ow[ _]Commons.*?
[Vv]fd.*?
[Dd]elete
[Qq]\-delete
[Dd]b\-\w+?.*?
[Nn]o[ _]source.*?
[Nn]o[ _]license.*?
[Nn]ostatus.*?
[Бб]ез[ _]ліцензії.*?
[Бб]ез[ _]джерела.*?
'''

# Користувач:BotCat/Без шаблона
no_template = '''
GFDL
GFDL\-self
GNU[ _]FDL
GNU[ _]Free[ _]Documentation[ _]License
GPL
FDL
PD\-user
PD\-self
PD\-US
PD
PU
[Сс]успільне[ _]надбання
PD\-Art
PD\-USGov
PD\-USGov\-CIA\-WF
PD\-USGov\-NARA
PD\-USGov\-NASA
LGPL
FAL
CC(\-BY)?(\-SA)?
[Kk]remlin.ru\-image
[Ff]ree[ _]?use
PD\-Poland
fair[ _]?use
[Aa]rt
[Bb]ook[ _]cover
[Оо]бкладинка[ _]книги
[Тт]итульний[ _]аркуш
[Mm]agazine[ _]cover
[Aa]lbumcover
[Dd]vdtapecover
[Оо]бкладинка[ _]dvd
[Vv]ideopcgame
[Ll]ogo
[Лл]оготип
[Кк]адр
[Сс]кр[іи]ншот
[Зз]німое[ _]екран[ау] 
[Ss]creenshot
DVD[ _]Cover
[Pp]oster
[Pp]olitposter
[Пп]остер
[Гг]ерби?
[Пп]рапори?
[Іі]сторичний[ _]плакат
[Mm]oney
[Ss]tamp
Creative[ _]Commons
Attribution
Share\-alike
ShareAlike
'''



#!/usr/bin/python3

""" 
Get a link to wiki page (as parameter, or interactively)
and returns a markup optimized for wordpress:
1. Replaced wiki latex generated images to native wordpress $latex $ tags.
2. Removes [.edit] links.
3. Points links to right domain (from which was get page).
4. Removes red links.
5. Replaces geshi highlighting to [sourcecode lang=""] tags.

Depends on bs4 unit.
"""
__author__ = "Bunyk T."

import re
import sys
import httplib2
import bs4

http = httplib2.Http()

def get(url):
    return http.request(url)[1].decode('utf-8')

def get_source(domain, name):
    text = get(domain + '/w/index.php?title=' + name + '&action=raw')
    sources = re.findall('<source lang="(.*?)">(.*?)</source>', text, re.DOTALL)
    return sources

def wiki_filter(ln, domain):
    ln = re.sub( # replace latex
        r'<img (?:(?:class="tex"|alt="(.*?)"|src=".*?") ?){3}/>',
        r'$latex \1$',
    ln)
    ln = re.sub( # remove [.edit] links
        r'<span class="editsection">.*?</span>',
        "",
    ln)
    ln = re.sub( # remove comments
        r'<!--.*?-->',
        r"",
    ln)
    ln = re.sub( # point links to right domain
        r'<a href="/wiki/(.*?)"',
        r'<a href="' + domain + r'wiki/\1"',
    ln)
    return ln

START_MARK = "<!-- bodycontent -->"
FINISH_MARK = "<!-- /bodycontent -->"
GESHI_MARK = "GESHI-HIGHLIGHT"

if len(sys.argv) < 2:
    url = input("Page url: ")
else:
    url = sys.argv[1]

try:
    domain = re.findall("(http://.*?/)wiki/", url)[0]
    page_name = re.findall("http://.*?/wiki/(.*)$", url)[0]
except IndexError:
    domain = re.findall("(http://.*?/)w/index.php", url)[0]
    page_name = re.findall("http://.*?/w/index.php.*title=(.*?)&.*", url)[0]

doc = get(url)

has_source = False

soup = bs4.BeautifulSoup(doc)
geshis = soup.findAll("div", {"class": "mw-geshi"})
for i in geshis: # replace geshi highlight to temporal mark
    i.replaceWith(GESHI_MARK)
    has_source = True

red_links = soup.findAll("a", {"class":"new"}) # Remove red links
for i in red_links:
    i.replaceWith(bs4.BeautifulSoup(i.renderContents()))

if has_source:
    sources_list = get_source(domain, page_name) # get sources used on page
current_source = 0

doc = str(soup).splitlines()

inbody = False
for line in doc:
    if re.search(FINISH_MARK, line):
        break
    if re.search(GESHI_MARK, line):
        print(''.join([
            "[sourcecode language='",
            sources_list[current_source][0],
            "']",
            sources_list[current_source][1],
            "[/sourcecode]",
        ]))
        current_source += 1
        continue
    if inbody:
        print(wiki_filter(line, domain))
        continue
    if re.search(START_MARK, line):
        inbody = True



from collections import defaultdict

from bwikibot.ui import render

template = r"""
{{Фільм
| українська назва  = ${title_uk}
| оригінальна назва = ${title}
| плакат            = [[${poster_file}|200px]]
| режисер           = ${director|link}
| продюсер          = ${producer|link}
| сценарист         = ${written_by|link}
| актори            = ${actors|link_list}
| кінокомпанія      = ${distributors|link_list}
| країна            = ${countries|link_list}
| рік               = ${year}
| мова              = ${languages|link_list}
| тривалість        = ${running_minutes} хв.
| кошторис          = ${budget}
| касові збори      = ${box_office}
| ідентифікатор     = ${imdb_id}
}}
'''${title_uk}''' ({{lang-${lang_code}|${title}}}) - фільм.

== В ролях ==
% for actor in actors:
* ${actor|link}
% endfor

== Посилання ==
% if imdb_id:
* {{Imdb title|${imdb_id}|${title}}} 
% endif

${link_list(categories, separator='\n')}

${interwikis}
"""

info = defaultdict(lambda :'',
    title_uk='Хмарний Атлас',
    title='Cloud Atlas',
    poster_file='Файл:example.jpg',
    director='director',
    producer='producer',
    written_by='сценарист',
    actors=['Том Хенкс', 'Холлі Бері'],
    distributors=['Warner brothers'],
    countries=['США'],
    year=2012,
    languages=[('Англійська мова', 'Англійська')],
    running_minutes=172,
    budget='102000000',
    box_office='102 млн.',
    imdb_id=1371111,
    lang_code='en',
    categories=['Категорія:Фільми {}'.format(2012),],
    interwikis='[[en:Cloud Atlas]]',
)

if __name__ == '__main__':
    print(render(template, **info))
